import React, { Component, PropTypes } from 'react'
import styled, { css, keyframes } from 'styled-components'
import Pane from './Pane'
import Overlay from './Overlay'
import Close from './Close'

const SIDES = [
  'top',
  'right',
  'bottom',
  'left'
]

const slideInAnimation = keyframes`
  from {
    transform: translateX(100%);
  }
  to {
    transform: translateX(0);
  }
`
const slideOutAnimation = keyframes`
  from {
    transform: translateX(0);
  }
  to {
    transform: translateX(100%);
  }
`

const styles = ({ slideOut, theme }) => css`
  transform: translateZ(0);
  animation: ${slideOut ? slideOutAnimation : slideInAnimation } ${theme.animationDuration.slow} ${theme.animationEasing.standard};
`

const SheetPane = styled(({ width, side, slideOut, ...props }) =>
  <Pane
    white
    position='absolute'
    shadow
    width={width}
    top={side !== 'bottom' ? 0 : undefined}
    right={side !== 'left' ? 0 : undefined}
    bottom={side !== 'top' ? 0 : undefined}
    left={side !== 'right' ? 0 : undefined}
    {...props} />)`${styles}`

SheetPane.propTypes = {
  width: PropTypes.string,
  side: PropTypes.oneOf(SIDES),
  slideOut: PropTypes.bool
}

SheetPane.displayName = 'SheetPane'

class Sheet extends Component {
  constructor() {
    super()

    this.handleClose = this.handleClose.bind(this)
    this.handleAnimationEnd = this.handleAnimationEnd.bind(this)

    this.state = {
      isClosing: false
    }
  }

  handleClose() {
    this.setState({ isClosing: true })
  }

  handleAnimationEnd() {
    if (!this.state.isClosing) return
    this.props.onClose()
    this.setState({ isClosing: false })
  }

  render() {
    const { onClose, isOpen, ...props } = this.props
    const { isClosing } = this.state

    if (!isOpen) return null

    return (
      <Overlay onClick={this.handleClose} fadeOut={isClosing} onAnimationEnd={this.handleAnimationEnd}>
        <SheetPane slideOut={isClosing} {...props} />
      </Overlay>
    )
  }
}

Sheet.propTypes = {
  width: PropTypes.string,
  side: PropTypes.oneOf(SIDES),
  isOpen: PropTypes.bool,
  onClose: PropTypes.func
}

Sheet.defaultProps = {
  width: '680px',
  side: 'right',
  isOpen: true
}


Sheet.displayName = 'Sheet'

export default Sheet
